<?php


    class Ragtek_PC_ControllerPublic_PageEdit extends
        XenForo_ControllerPublic_Abstract
    {

        protected $_templateDeleteConfirm = 'ragtek_pc_delete_confirm';

        public function actionIndex()
        {
            $postId = $this->_input->filterSingle('comment_id', XenForo_Input::UINT);

            $comment = $this->getCommentData($postId);
            if ($comment) {
                return $this->getCommentSpecificRedirect($comment);
            }
        }

        public function actionChangeCommentState()
        {

            $page = $this->_input->filterSingle('node_name', XenForo_Input::STRING);
            $page = $this->getModelFromCache('XenForo_Model_Page')->getPageByName($page);

            if ($this->isConfirmedPost()) {
                $this->getModelFromCache('Ragtek_PC_Model_Pagecomment')->changePageCommentState($page);
                $link = XenForo_Link::buildPublicLink('pages', $page);
                return $this->responseRedirect(XenForo_ControllerResponse_Redirect::SUCCESS, $link);
            }

            $viewParams = array(
                'page' => $page
            );
            return $this->responseView('', 'ragtek_page_changestate', $viewParams);
        }

        public function actionDelete()
        {
            $postId = $this->_input->filterSingle('comment_id', XenForo_Input::UINT);
            $comment = $this->_getCommentModel()->getCommentById($postId,
                                                                 array('join' => Ragtek_PC_Model_Pagecomment::FETCH_NODE)
            );

            if (!$this->_getCommentModel()->canDeleteComment($comment)) {
                return $this->responseNoPermission();
            }

            if ($this->isConfirmedPost()) {

                $this->_getCommentModel()->deleteCommentById($comment['comment_id']);
                $link = XenForo_Link::buildPublicLink('pages', $comment['node_name']);
                return $this->responseRedirect(
                    XenForo_ControllerResponse_Redirect::SUCCESS,
                    $link
                );
            }

            $viewParams = array(
                'comment' => $comment
            );
            $link = XenForo_Link::buildPublicLink('page-comment/delete', $comment);

            $params = Ragtek_PC_Helper::getViewFormParams($link, 'delete comment', Ragtek_PC_Helper::DEL_TYPE_ONLY_HARD);
            $viewParams += $params;
            return $this->responseView('Ragtek_PC_View_Delete', $this->_templateDeleteConfirm, $viewParams);
        }

        public function actionQuote()
        {
            $postId = $this->_input->filterSingle('comment_id', XenForo_Input::UINT);


            $comment = $this->_getCommentModel()->getCommentById($postId, array(
                                                                               'join' => Ragtek_PC_Model_Pagecomment::FETCH_USER
                                                                          ));
            $quote = $this->_getCommentModel()->getQuoteTextForComment($comment);
            $viewParams = array(
                'post' => $comment,
                'quote' => $quote
            );

            return $this->responseView('XenForo_ViewPublic_Post_Quote', 'post_quote', $viewParams);
        }

        public function actionReply()
        {

            return $this->responseMessage('Sorry, you need enabled javascript for this ATM');
            $page = $this->_input->filterSingle('node_name', XenForo_Input::STRING);
            $page = $this->getModelFromCache('XenForo_Model_Page')->getPageByName($page);

            $postId = $this->_input->filterSingle('quote', XenForo_Input::UINT);
            $comment = $this->_getCommentModel()->getCommentById($postId, array(
                                                                               'join' => Ragtek_PC_Model_Pagecomment::FETCH_USER
                                                                          ));
            $quote = $this->_getCommentModel()->getQuoteTextForComment($comment);
            $viewParams = array(
                'post' => $comment,
                'quote' => $quote
            );

            # return $this->responseView('', '')


        }


        public function actionEditPage()
        {
            $this->_checkPermission();
            $page = $this->_input->filterSingle('node_name', XenForo_Input::STRING);
            $page = $this->getModelFromCache('XenForo_Model_Page')->getPageByName($page);

            $templateModel = $this->getModelFromCache('XenForo_Model_Template');

            $template = $templateModel->getTemplateInStyleByTitle($this->getModelFromCache('XenForo_Model_Page')->getTemplateTitle($page));


            $viewParams = array(
                'page' => $page,
                'template' => $template
            );
            return $this->responseView('', 'ragtek_page_pageeditor', $viewParams);
        }

        public function actionSavePage()
        {
            $this->_checkPermission();
            $this->_assertPostOnly();


            $pageData = $this->_input->filter(array(
                                                   'title' => XenForo_Input::STRING,
                                                   'node_name' => XenForo_Input::STRING,
                                              ));

            $template = $this->_input->filterSingle('template', XenForo_Input::STRING);

            $page = $this->_input->filterSingle('node_name', XenForo_Input::STRING);
            $page = $this->getModelFromCache('XenForo_Model_Page')->getPageByName($page);

            $nodeId = $page['node_id'];
            $pageData['modified_date'] = XenForo_Application::$time;
            $nodeId = $this->getModelFromCache('XenForo_Model_Page')->savePage(
                $pageData,
                $template,
                $nodeId,
                $this->_input->filterSingle('template_id', XenForo_Input::UINT)
            );

            $link = XenForo_Link::buildPublicLink('pages', $page);
            return $this->responseRedirect(XenForo_ControllerResponse_Redirect::SUCCESS, $link);

        }

        public function actionEditInline()
        {
            $postId = $this->_input->filterSingle('comment_id', XenForo_Input::UINT);

            $comment = $this->getCommentData($postId);

            $link = XenForo_Link::buildPublicLink('page-comment/save-inline', $comment);

            $params = Ragtek_PC_Helper::getViewFormParams($link, 'edit comment', Ragtek_PC_Helper::DEL_TYPE_ONLY_HARD);
            $params += array('post' => $comment);
            return $this->responseView('Ragtek_PC_View_EditInline', 'ragtek_pc_inlineeditor', $params);
        }


        public function actionIp()
        {
            $commentId = $this->_input->filterSingle('comment_id', XenForo_Input::UINT);
            $comment = $this->getCommentData($commentId);

            if (!$comment['ip_id']) {
                return $this->responseError(new XenForo_Phrase('no_ip_information_available'));
            }

            $viewParams = array(
                'title' => new XenForo_Phrase('ragtek_pc_ipdata_for_x', array('name' => 'comment')),
                'ipInfo' => $this->getModelFromCache('XenForo_Model_Ip')->getContentIpInfo($comment)
            );

            return $this->responseView('', 'ragtek_pc_ip_info', $viewParams);
        }

        public function actionEdit()
        {
            $postId = $this->_input->filterSingle('comment_id', XenForo_Input::UINT);

            $comment = $this->getCommentData($postId);

            if ($this->_input->inRequest('more_options')) {
                $comment['message'] = $this->getHelper('Editor')->getMessageText('message', $this->_input);
            }

            $viewParams = array(
                'post' => $comment,
                'canDeletePost' => true,
                'canHardDeletePost' => true
            );

            return $this->responseView('Ragtek_PC_View_EditInline', 'ragtek_page_commentedit', $viewParams);
        }

        public function actionSaveInline()
        {
            $this->_assertPostOnly();

            if ($this->_input->inRequest('more_options')) {
                return $this->responseReroute(__CLASS__, 'edit');
            }

            $postId = $this->_input->filterSingle('comment_id', XenForo_Input::UINT);


            $post = $this->getCommentData($postId);

            $message = XenForo_Helper_String::autoLinkBbCode(
                $this->getHelper('Editor')->getMessageText('message', $this->_input));


            $dw = XenForo_DataWriter::create('Ragtek_PC_DW_Comments');
            $dw->setExistingData($postId);
            $dw->set('message', $message);

            $dw->save();

            return $this->getCommentSpecificRedirect($post);
        }


        protected function getCommentData($commentId)
        {
            $fetch = array('join' => Ragtek_PC_Model_Pagecomment::FETCH_NODE |
                                     Ragtek_PC_Model_Pagecomment::FETCH_USER);
            return $this->_getCommentModel()->getCommentById($commentId, $fetch);

        }

        /**
         * @return Ragtek_PC_Model_Pagecomment
         */
        protected function _getCommentModel()
        {
            return $this->getModelFromCache('Ragtek_PC_Model_Pagecomment');
        }


        public function getCommentSpecificRedirect(array $comment, $redirectType = XenForo_ControllerResponse_Redirect::SUCCESS)
        {
            $comment['position'] = $this->_getCommentModel()->getCommentBeforeOnPage($comment);

            return $this->responseRedirect(
                $redirectType,
                XenForo_Link::buildPublicLink('pages/page-permalink', $comment, array('pagecomment' => $comment))
            );
        }

        protected function _checkPermission()
        {
            if (!XenForo_Visitor::getInstance()->hasAdminPermission('node')) {
                throw $this->responseException($this->responseNoPermission());
            }
        }
    }
 
