<?php

    class Ragtek_PC_DW_Comments extends
        XenForo_DataWriter
    {

        const OPTION_SET_IP_ADDRESS = 'setIpAddress';

        protected function _getUpdateCondition($tableName)
        {
            return 'comment_id = ' . $this->_db->quote($this->getExisting('comment_id'));
        }


        protected function _getExistingData($data)
        {
            if (!$commentId = $this->_getExistingPrimaryKey($data, 'comment_id')) {
                return false;
            }

            return array('ragtek_pagecomments' => $this->_getCommentModel()->getCommentById($commentId));
        }

        protected function _getDefaultOptions()
        {
            return array(
                self::OPTION_SET_IP_ADDRESS => true

            );
        }

        protected function _getFields()
        {
            return array(
                'ragtek_pagecomments' => array(

                    'comment_id' => array('type' => self::TYPE_UINT, 'autoIncrement' => true),
                    'node_id' => array('type' => self::TYPE_UINT, 'required' => true),
                    'user_id' => array('type' => self::TYPE_UINT, 'required' => true),
                    'post_date' => array('type' => self::TYPE_UINT, 'required' => true, 'default' => XenForo_Application::$time),
                    'message' => array('type' => self::TYPE_STRING, 'required' => true,
                                       'requiredError' => 'please_enter_valid_message',

                    ),
                    'ip_id' => array('type' => self::TYPE_UINT, 'default' => 0),
                )
            );
        }

        protected function _postSave()
        {
            $comment = $this->getMergedData();
            $this->_getCommentModel()->alertQuotedMembers($comment);
            $this->_insertOrUpdateSearch();

            if ($this->isInsert() && $this->getOption(self::OPTION_SET_IP_ADDRESS) && !$this->get('ip_id')) {

                $this->_updateIpData();
            }

            if ($this->isInsert()) {
                $user = XenForo_Model::create('XenForo_Model_User');
                $user = $user->getUserById($comment['user_id']);
                $this->getModelFromCache('XenForo_Model_NewsFeed')->publish(
                    $comment['user_id'], $user['username'], $this->getContentType(), $comment['comment_id'], 'insert'
                );
            }
        }

        protected function _updateIpData()
        {
            $commentId = $this->get('comment_id');

            $ipId = XenForo_Model_Ip::log(
                $this->get('user_id'), $this->getContentType(), $commentId, 'insert'
            );
            $this->set('ip_id', $ipId, '', array('setAfterPreSave' => true));

            $this->_db->update('ragtek_pagecomments', array(
                                                           'ip_id' => $ipId
                                                      ), 'comment_id' . ' = ' . $this->_db->quote($commentId));
        }

        protected function _postDelete()
        {
            $this->_deleteSearch();
        }

        protected function _insertOrUpdateSearch()
        {
            $dh = $this->_getSearchHandler();
            $indexer = new XenForo_Search_Indexer();
            $data = $this->getMergedData();

            $dh->insertIntoIndex($indexer, $data);

        }

        protected function _deleteSearch()
        {
            $dh = $this->_getSearchHandler();
            $indexer = new XenForo_Search_Indexer();
            $dh->deleteFromIndex($indexer, $this->getMergedData());
        }

        protected function _deleteIP()
        {
            $this->getModelFromCache('XenForo_Model_Ip')->deleteByContent(
                $this->getContentType(), $this->get('comment_id')
            );
        }

        /**
         * @return Ragtek_PC_Model_Pagecomment
         */
        protected function _getCommentModel()
        {
            return $this->getModelFromCache('Ragtek_PC_Model_Pagecomment');
        }

        protected function _getSearchHandler()
        {
            return $this->getModelFromCache('XenForo_Model_Search')->getSearchDataHandler($this->getContentType());
        }

        public function getContentType()
        {
            return Ragtek_PC_StaticMethods::CONTENT_TYPE;
        }

    }