<?php

    class Ragtek_PC_Model_Pagecomment extends
        XenForo_Model
    {
        const FETCH_USER = 0x01;
        const FETCH_NODE = 0x02;
        const FETCH_USER_OPTIONS = 0x04;
        const FETCH_AVATAR = 0x08;


        public function canReply($nodeId)
        {
            $visitor = XenForo_Visitor::getInstance();
            if ($visitor['user_id'] != 0) {
                return true;
            }
            return false;
        }

        public function insertComment($comment)
        {
            $writer = $this->_getCommentDW();
            $writer->set('user_id', $comment['user_id']);
            $writer->set('message', $comment['message']);
            $writer->set('node_id', $comment['node_id']);
            $writer->save();
        }

        public function getNextCommentInPage($nodeId, $postDate, array $fetchOptions = array())
        {
            $db = $this->_getDb();

            return $db->fetchRow($db->limit('
			SELECT *
			FROM ragtek_pagecomments AS comments
			WHERE comments.node_id = ?
			AND
			comments.post_date < ?
			ORDER BY comments.post_date
		', 1), array($nodeId, $postDate));
        }

        public function commentsAllowed(array &$page)
        {

            if (!isset($page['ragtek_allow_comments'])) {
                $pageData = $this->_getPageModel()->getPageById($page['node_id']);
                $page['ragtek_allow_comments'] = $pageData['ragtek_allow_comments'];
            }
            if (isset($page['ragtek_allow_comments']) AND $page['ragtek_allow_comments'] == 1) {
                return true;
            }
            else
            {
                return false;
            }
        }


        public function changePageCommentState(array $page)
        {
            /** @var $pageDW XenForo_DataWriter_Page */
            $pageDW = XenForo_DataWriter::create('XenForo_DataWriter_Page');
            $pageDW->setExistingData($page);
            if ($page['ragtek_allow_comments']) {
                $pageDW->set('ragtek_allow_comments', 0);
            }
            else {
                $pageDW->set('ragtek_allow_comments', 1);
            }
            $pageDW->save();
            return true;
        }

        /**
         * @param array $node
         * @return string
         */
        public function getCommentBeforeOnPage(array $node)
        {
            $nodeId = $node['node_id'];
            $date = $node['post_date'];
            $commentsBefore = $this->_getDb()->fetchOne('
			SELECT COUNT(*)
			FROM ragtek_pagecomments AS comments
			WHERE comments.node_id = ?
			AND 
			comments.post_date < ?
		', array($nodeId, $date)
            );
            return $commentsBefore;
        }


        public function getCommentIdsInRange($lastId, $batchSize)
        {
            $db = $this->_getDb();
            return $db->fetchCol($db->limit('
			SELECT comment_id
			FROM ragtek_pagecomments
			WHERE comment_id > ?
			ORDER BY comment_id
		', $batchSize), $lastId);
        }

        public function getCommentsByUserId($userId)
        {
            $query = "
        SELECT *
        FROM ragtek_pagecomments
        WHERE user_id = ?
        ";

            return $this->fetchAllKeyed($query, 'comment_id', $userId);
        }

        public function getCommentsByIds($ids, array $fetch = array())
        {

            $join = $this->prepareCommentFetchOptions($fetch);

            $query = '
        SELECT comment.*
	' . $join['selectFields'] . '
        FROM ragtek_pagecomments AS comment
       ' . $join['joinTables'] . '
       WHERE comment.comment_id IN (' . $this->_getDb()->quote($ids) . ')';

            return $this->fetchAllKeyed($query, 'comment_id');
        }

        public function getCommentsByPageId($node_id, array $fetch = array())
        {

            if (count($fetch) == 0) {

                $perPage = 1000000;
                $page = 1;
            }
            else {
                $perPage = $fetch['perPage'];
                $page = $fetch['page'];

                if ($page != 1) {
                    self::$position = $perPage * ($page - 1);
                }
            }
            $limitOptions = $this->prepareLimitFetchOptions($fetch);
            $join = $this->prepareCommentFetchOptions($fetch);

            $order = $join['orderClause'];

            $query = '
        SELECT comment.*
	' . $join['selectFields'] . '
        FROM ragtek_pagecomments AS comment
       ' . $join['joinTables'] . '
        WHERE comment.node_id =  ' . $this->_getDb()->quote($node_id) . '
        ORDER BY comment.comment_id
        ';

            $comments = $this->_getDb()->fetchAll($this->limitQueryResults($query, $limitOptions['limit'], $limitOptions['offset']));

            foreach ($comments AS &$comment) {
                $this->getPermissions($comment);

            }
            return $comments;
        }

        public function getCommentById($comment_id, array $fetch = array())
        {
            $join = $this->prepareCommentFetchOptions($fetch);

            $query = '
        SELECT comment.*
	' . $join['selectFields'] . '
	 FROM ragtek_pagecomments AS comment
       ' . $join['joinTables'] . '
        WHERE comment_id = ?';

            $comment = $this->_getDb()->fetchRow($query, $comment_id);
            if (!$comment) {
                return false;
            }
            return $comment;
        }

        public function getNewestCommentsByDate($nodeId, $date)
        {
            $fetch = array(
                'join' => Ragtek_PC_Model_Pagecomment::FETCH_USER | Ragtek_PC_Model_Pagecomment::FETCH_AVATAR,
                'limit' => 5
            );

            $join = $this->prepareCommentFetchOptions($fetch);
            $limitOptions = $this->prepareLimitFetchOptions($fetch);
            $query = '
         SELECT comment.*
     ' . $join['selectFields'] . '
         FROM ragtek_pagecomments AS comment
        ' . $join['joinTables'] . '
         WHERE comment.node_id =  ' . $this->_getDb()->quote($nodeId) . '
         AND
         comment.post_date > ?
         ORDER BY comment.comment_id DESC
         ';

            $comments = $this->_getDb()->fetchAll($this->limitQueryResults(
                                                      $query,
                                                      $limitOptions['limit'], $limitOptions['offset']), $date);

            foreach ($comments AS &$comment) {
                $this->getPermissions($comment);
            }

            return $comments;
        }


        public function countCommentsForPage($node_id)
        {
            return $this->_getDb()->fetchOne('
			SELECT COUNT(*)
			FROM ragtek_pagecomments AS comments
			WHERE node_id = ?
		', $node_id);
        }


        public function deleteCommentById($commentId)
        {
            $dw = $this->_getCommentDW();
            $dw->setExistingData($commentId);
            $dw->delete();
        }


        public function canViewComment(array $comment, &$errorPhraseKey = '', array $nodePermissions = null, array $viewingUser = null)
        {
            $visitor = XenForo_Visitor::getInstance();
            if (isset($comment['node_permission_cache'])) {
                $visitor->setNodePermissions($comment['node_id'], $comment['node_permission_cache']);
                unset($comment['node_permission_cache']);
            }

            if (!$this->_getPageModel()->canViewPage($comment, $errorPhraseKey)) {
                return false;
            }
            // TODO check other things once available (moderated, etc..)
            return true;
        }

        public function prepareCommentFetchOptions(array $fetchOptions)
        {
            $selectFields = '';
            $joinTables = '';
            $orderBy = '';

            if (!empty($fetchOptions['order'])) {
                $orderBySecondary = '';

                switch ($fetchOptions['order'])
                {
                    case 'title':
                    case 'position':
                        $orderBy = 'comment.' . $fetchOptions['order'];
                        break;

                    default:
                        $orderBy = 'comment.post_date';
                }
                if (!isset($fetchOptions['orderDirection']) || $fetchOptions['orderDirection'] == 'desc') {
                    $orderBy .= ' DESC';
                }
                else
                {
                    $orderBy .= ' ASC';
                }

                $orderBy .= $orderBySecondary;
            }

            if (!empty($fetchOptions['join'])) {

                if ($fetchOptions['join'] & self::FETCH_USER) {

                    $selectFields .= ',
					user.* ';
                    $joinTables .= '
					LEFT JOIN xf_user AS user ON
						(user.user_id = comment.user_id)';
                }
                else {
                    if ($fetchOptions['join'] & self::FETCH_AVATAR) {
                        $selectFields .= ',
					user.avatar_date, user.gravatar';
                        $joinTables .= '
					LEFT JOIN xf_user AS user ON
						(user.user_id = comment.user_id)';
                    }
                }
                if ($fetchOptions['join'] & self::FETCH_USER_OPTIONS) {
                    $selectFields .= ',
                                user_option.*';
                    $joinTables .= '
                                LEFT JOIN xf_user_option AS user_option ON
                                    (user_option.user_id = comment.user_id)';
                }


                if ($fetchOptions['join'] & self::FETCH_NODE) {
                    $selectFields .= ',
					node.title AS node_name2, node.node_name AS node_name';

                    $joinTables .= '
					INNER JOIN xf_node AS node ON
						(node.node_id = comment.node_id)';
                }
            }
            // FOR FUTURE
            /*
              if (isset($fetchOptions['likeUserId'])) {
                  if (empty($fetchOptions['likeUserId'])) {
                      $selectFields .= ',
                          0 AS like_date';
                  }
                  else
                  {
                      $selectFields .= ',
                          liked_content.like_date';
                      $joinTables .= '
                          LEFT JOIN xf_liked_content AS liked_content
                              ON (liked_content.content_type = \'' . Ragtek_AS_Constants::CONTENT_TYPE . '\'
                                  AND liked_content.content_id = article.article_id
                                  AND liked_content.like_user_id = ' . $this->_getDb()->quote($fetchOptions['likeUserId']) . ')';
                  }
              }
            */

            if (!empty($fetchOptions['permissionCombinationId'])) {
                $selectFields .= ',
				permission.cache_value AS node_permission_cache';
                $joinTables .= '
				LEFT JOIN xf_permission_cache_content AS permission
					ON (permission.permission_combination_id = ' . $this->_getDb()->quote($fetchOptions['permissionCombinationId']) . '
						AND permission.content_type = \'node\'
						AND permission.content_id = comment.node_id)';
            }


            return array(
                'selectFields' => $selectFields,
                'joinTables' => $joinTables,
                'orderClause' => ($orderBy ? "ORDER BY $orderBy" : '')
            );
        }

        static $position = 0;

        public function getPermissions(&$comment)
        {
            $this->standardizeViewingUserReferenceForNode($comment['node_id'], $viewingUser, $nodePermissions);
            $comment['position'] = self::$position;
            $comment['canEdit'] = $this->canEditComment($comment, $null, $nodePermissions);
            $comment['canReply'] = $this->canReply($comment['node_id']);
            $comment['canLike'] = false;
            $comment['canDelete'] = $this->canDeleteComment($comment, $null, $nodePermissions);
            $comment['canSeeIp'] = $viewingUser['is_admin'];
            self::$position++;
        }

        public function canDeleteComment(array $comment, &$errorPhraseKey = '', array $nodePermissions = null, array $viewingUser = null)
        {
            $this->standardizeViewingUserReferenceForNode($comment['node_id'], $viewingUser, $nodePermissions);

            // ALLOW ALL ADMINS DELETING
            if ($viewingUser['is_admin']) {
                return true;
            }

            if ($viewingUser['user_id'] == $comment['user_id']) {
                return true;
            }
            return false;
        }

        public function canEditComment(array $comment, &$errorPhraseKey = '', array $nodePermissions = null, array $viewingUser = null)
        {
            $this->standardizeViewingUserReferenceForNode($comment['node_id'], $viewingUser, $nodePermissions);

            if ($viewingUser['is_admin']) {
                return true;
            }

            if ($viewingUser['user_id'] == $comment['user_id']) {
                return true;
            }
            return false;


        }

        public function getQuoteTextForComment(array $comment, $maxQuoteDepth = 0)
        {

            return '[quote="' . $comment['username'] . ', pagecomment: ' . $comment['comment_id'] . '"]'
                   . trim(XenForo_Helper_String::stripQuotes($comment['message'], $maxQuoteDepth))
                   . "[/quote]\n";
        }

        /**
         * Sends an alert to members directly quoted in a post
         *
         * @param array $comment
         */
        public function alertQuotedMembers(array $comment)
        {
            $quotedUsers = array();

            if (preg_match_all('/\[quote=("|\'|)([^,]+),\s*pagecomment:\s*(\d+)\\1\]/siU', $comment['message'], $quotes)) {
                $fetch = array(
                    'join' => Ragtek_PC_Model_Pagecomment::FETCH_USER |
                              Ragtek_PC_Model_Pagecomment::FETCH_USER_OPTIONS
                );

                $quotedComments = $this->getCommentsByIds(array_unique($quotes[3]), $fetch);


                foreach ($quotedComments AS $quotedCommentId => $quotedComment)
                {
                    if (!isset($quotedUsers[$quotedComment['user_id']]) && $quotedComment['user_id'] && $quotedComment['user_id'] != $comment['user_id']) {

                        if (XenForo_Model_Alert::userReceivesAlert($quotedComment, Ragtek_PC_StaticMethods::CONTENT_TYPE, 'quote')) {
                            $quotedUsers[$quotedComment['user_id']] = true;

                            XenForo_Model_Alert::alert($quotedComment['user_id'],
                                                       $comment['user_id'], $quotedComment['username'],
                                                       Ragtek_PC_StaticMethods::CONTENT_TYPE, $comment['comment_id'],
                                                       'quote'
                            );
                        }
                    }
                }
            }

            return array_keys($quotedUsers);
        }

        /**
         * @return XenForo_Model_Page
         */
        protected function _getPageModel()
        {
            return $this->getModelFromCache('XenForo_Model_Page');
        }

        /**
         * @return Ragtek_PC_DW_Comments
         */
        protected function _getCommentDW()
        {
            return XenForo_DataWriter::create('Ragtek_PC_DW_Comments');
        }
    }
 
