<?php

    class Ragtek_PC_Search_DataHandler_Comment extends XenForo_Search_DataHandler_Abstract
    {
        /**
         * Get the content types that will be searched, when doing a type-specific search for this type.
         * This may be multiple types (for example, thread and post for post searches).
         *
         * @return array
         */
        public function getSearchContentTypes()
        {
            return array(Ragtek_PC_StaticMethods::CONTENT_TYPE);
        }

        /**
         * Render a result (as HTML).
         *
         * @param XenForo_View $view
         * @param array $result Data from result
         * @param array $search The search that was performed
         *
         * @return XenForo_Template_Abstract|string
         */
        public function renderResult(XenForo_View $view, array $result, array $search)
        {

            return $view->createTemplateObject('ragtek_page_searchresult', array(
                                                                                'post' => $result));
        }

        /**
         * Gets the date of the result (from the result's content).
         *
         * @param array $result
         *
         * @return integer
         */
        public function getResultDate(array $result)
        {
            return $result['post_date'];
        }

        /**
         * Determines if the specific search result (data from getDataForResults()) can be viewed
         * by the given user. The user and combination ID will be the same as given to getDataForResults().
         *
         * @param array $result Data for a result
         * @param array $viewingUser Information about the viewing user (keys: user_id, permission_combination_id, permissions)
         *
         * @return boolean
         */
        public function canViewResult(array $result, array $viewingUser)
        {
            #  return true;
            return $this->_getPageCommentModel()->canViewComment($result, $null, $result['permissions'], $viewingUser);
        }

        /**
         * Gets the additional, type-specific data for a list of results. If any of
         * the given IDs are not returned from this, they will be removed from the results.
         *
         * @param array $ids List of IDs of this content type.
         * @param array $viewingUser Information about the viewing user (keys: user_id, permission_combination_id, permissions)
         * @param array $resultsGrouped List of all results grouped by content type
         *
         * @return array Format: [id] => data, IDs not returned will be removed from results
         */
        public function getDataForResults(array $ids, array $viewingUser, array $resultsGrouped)
        {
            $result = $this->_getPageCommentModel()->getCommentsByIds($ids,
                                                                      array(
                                                                           'join' => Ragtek_PC_Model_Pagecomment::FETCH_NODE |
                                                                                     Ragtek_PC_Model_Pagecomment::FETCH_USER |
                                                                                     Ragtek_PC_Model_Pagecomment::FETCH_AVATAR,
                                                                           'permissionCombinationId' => $viewingUser['permission_combination_id']
                                                                      ));

            foreach ($result AS &$comment) {
                $comment['position'] = $this->_getPageCommentModel()->getCommentBeforeOnPage($comment);
            }

            return $this->_getPageCommentModel()->unserializePermissionsInList($result, 'node_permission_cache');

        }

        /**
         * Indexes the specified content IDs.
         *
         * @param XenForo_Search_Indexer $indexer
         * @param array $contentIds
         *
         * @return array List of content IDs indexed
         */
        public function quickIndex(XenForo_Search_Indexer $indexer, array $contentIds)
        {
            $comments = $this->_getPageCommentModel()->getCommentsByIds($contentIds);

            foreach ($comments AS $article)
            {

                $this->insertIntoIndex($indexer, $article);
            }

            return true;
        }

        /**
         * Rebuilds the index in bulk.
         *
         * @param XenForo_Search_Indexer $indexer Object that will will manipulate the index
         * @param integer $lastId The last ID that was processed. Should continue with the IDs above this.
         * @param integer $batchSize Number of records to process at once
         *
         * @return integer|false The last ID that was processed or false if none were processed
         */
        public function rebuildIndex(XenForo_Search_Indexer $indexer, $lastId, $batchSize)
        {
            $commentIds = $this->_getPageCommentModel()->getCommentIdsInRange($lastId, $batchSize);


            if (!$commentIds) {
                return false;
            }

            $this->quickIndex($indexer, $commentIds);

            return max($commentIds);
        }

        /**
         * Deletes one or more records from the index. Wrapper around {@link _deleteFromIndex()}.
         *
         * @param XenForo_Search_Indexer $indexer Object that will will manipulate the index
         * @param array $dataList A list of data to remove. Each element is an array of the data from one record.
         */
        protected function _deleteFromIndex(XenForo_Search_Indexer $indexer, array $dataList)
        {
            $articleIds = array();
            foreach ($dataList AS $data) {
                $articleIds = $data['comment_id'];
            }
            $indexer->deleteFromIndex(Ragtek_PC_StaticMethods::CONTENT_TYPE, $articleIds);
        }

        /**
         * Updates a record in the index.
         *
         * @param XenForo_Search_Indexer $indexer Object that will will manipulate the index
         * @param array $data Data that needs to be updated
         * @param array $fieldUpdates Key-value fields to update
         */
        protected function _updateIndex(XenForo_Search_Indexer $indexer, array $data, array $fieldUpdates)
        {
            $indexer->updateIndex(Ragtek_PC_StaticMethods::CONTENT_TYPE, $data['comment_id'], $fieldUpdates);
        }

        /**
         * Inserts a new record or replaces an existing record in the index.
         *
         * @param XenForo_Search_Indexer $indexer Object that will will manipulate the index
         * @param array $data Data that needs to be updated
         * @param array|null $parentData Data about the parent info (eg, for a post, the parent thread)
         */
        protected function _insertIntoIndex(XenForo_Search_Indexer $indexer, array $data, array $parentData = null)
        {
            $message = $data['message'];
            $indexer->insertIntoIndex(Ragtek_PC_StaticMethods::CONTENT_TYPE, $data['comment_id'], '',
                                      $message, $data['post_date'], $data['user_id'], $data['node_id'], array('node' => $data['node_id']));


        }


        public function getSearchFormControllerResponse(XenForo_ControllerPublic_Abstract $controller, XenForo_Input $input, array $viewParams)
        {
            $categoryId = $input->filterSingle('c', XenForo_Input::UINT);
            $pages = Ragtek_PC_Helper::getAllNodesByType('Page');

            foreach ($pages AS &$page) {
                if (!$this->_getPageCommentModel()->commentsAllowed($page)) {

                    unset($pages[$page['node_id']]);
                }
            }

            $viewParams['search'] = array_merge($viewParams['search'], array(
                                                                            'categories' => $pages,
                                                                            'category' => $categoryId));

            return $controller->responseView('', 'ragtek_page_searchform', $viewParams);
        }

        protected $_pageModel = null;

        /**
         * @return Ragtek_PC_Model_Pagecomment
         */
        protected function _getPageCommentModel()
        {
            if (!$this->_pageModel) {
                $this->_pageModel = XenForo_Model::create('Ragtek_PC_Model_Pagecomment');
            }
            return $this->_pageModel;
        }
    }
 
